@@ -36,7 +36,6 @@
 #include <errno.h>
 #include <glib/gi18n.h>
 #include <string.h>
-#include <webkit/webkit.h>
 
 G_DEFINE_TYPE (EphyDownload, ephy_download, G_TYPE_OBJECT)
 
@@ -140,9 +139,23 @@ ephy_download_set_property (GObject      *object,
   }
 }
 
+/**
+ * ephy_download_get_content_type:
+ * @download: an #EphyDownload
+ *
+ * Gets content-type information for @download. If the file is already
+ * present on the filesystem and readable, uses GIO to get the
+ * content-type. Otherwise it uses WebKit and Soup.
+ *
+ * Returns: content-type for @download, must be freed with g_free()
+ **/
 char *
 ephy_download_get_content_type (EphyDownload *download)
 {
+#ifdef HAVE_WEBKIT2
+  /* TODO: Downloads */
+  return NULL;
+#else
   WebKitNetworkResponse *response;
   SoupMessage *message;
   char *content_type = NULL;
@@ -182,49 +195,37 @@ ephy_download_get_content_type (EphyDownload *download)
   LOG ("ephy_download_get_content_type: %s", content_type);
 
   return content_type;
+#endif
 }
 
+
+/* Helper function to decide what EphyDownloadActionType should be the
+ * default for the download. This implies that you want something to
+ * happen, this function will never return EPHY_DOWNLOAD_ACTION_NONE.
+ */
 static EphyDownloadActionType
 decide_action_from_mime (EphyDownload *ephy_download)
 {
-  WebKitNetworkResponse *response;
-  SoupMessage *message;
-  char *mime_description = NULL;
+  char *content_type;
   GAppInfo *helper_app = NULL;
   EphyDownloadActionType action;
-  WebKitDownload *download;
 
-  download = ephy_download_get_webkit_download (ephy_download);
+  content_type = ephy_download_get_content_type (ephy_download);
+  if (content_type) {
+    helper_app = g_app_info_get_default_for_type (content_type, FALSE);
+    if (helper_app)
+      action = EPHY_DOWNLOAD_ACTION_OPEN;
 
-  response = webkit_download_get_network_response (download);
-  message = webkit_network_response_get_message (response);
-
-  if (message) {
-    char *content_type = ephy_download_get_content_type (ephy_download);
-
-    if (content_type) {
-      mime_description = g_content_type_get_description (content_type);
-      helper_app = g_app_info_get_default_for_type (content_type, FALSE);
-
-      if (helper_app)
-        action = EPHY_DOWNLOAD_ACTION_OPEN;
-
-      g_free (content_type);
-    }
+    g_free (content_type);
   }
 
-  if (mime_description == NULL) {
-    mime_description = g_strdup (C_("file type", "Unknown"));
-    action = EPHY_DOWNLOAD_ACTION_BROWSE_TO;
-  }
-
-  /* Sometimes downloads can have a mime_description but a NULL helper_app
-   * in that case action is never changed so DOWNLOAD_ACTION_DOWNLOAD remains
-   * as action value. This is the same response value as Save as...
-   * button, which is wrong for the Download button.
+  /* Downloads that have no content_type, or no helper_app, are
+   * considered unsafe/unable to open. Default them to BROWSE_TO.
    */
   if (helper_app == NULL)
     action = EPHY_DOWNLOAD_ACTION_BROWSE_TO;
+  else
+    g_object_unref (helper_app);
 
   return action;
 }
@@ -280,15 +281,13 @@ parse_extension (const char *filename)
 }
 
 static char *
-define_destination_uri (EphyDownload *download)
+define_destination_uri (EphyDownload *download, const char *suggested_filename)
 {
   char *dest_dir;
   char *dest_name;
   char *destination_filename;
   char *destination_uri;
-  const char *suggested_filename;
 
-  suggested_filename = webkit_download_get_suggested_filename (download->priv->download);
   dest_dir = ephy_file_get_downloads_dir ();
 
   /* Make sure the download directory exists */
@@ -372,7 +371,11 @@ ephy_download_set_destination_uri (EphyDownload *download,
 
   priv->destination = g_strdup (destination);
 
+#ifdef HAVE_WEBKIT2
+  /* TODO: Downloads */
+#else
   webkit_download_set_destination_uri (priv->download, priv->destination);
+#endif
   g_object_notify (G_OBJECT (download), "destination");
 }
 
@@ -385,14 +388,19 @@ ephy_download_set_destination_uri (EphyDownload *download,
 void
 ephy_download_set_auto_destination (EphyDownload *download)
 {
+#ifdef HAVE_WEBKIT2
+  /* In WebKit2 priv->destination == NULL means auto_destination */
+#else
   char *dest;
 
   g_return_if_fail (EPHY_IS_DOWNLOAD (download));
 
-  dest = define_destination_uri (download);
+  dest = define_destination_uri (download,
+                                 webkit_download_get_suggested_filename (download->priv->download));
   ephy_download_set_destination_uri (download, dest);
 
   g_free (dest);
+#endif
 }
 
 /**
@@ -598,8 +606,11 @@ ephy_download_start (EphyDownload *download)
 
   if (priv->destination == NULL)
     ephy_download_set_auto_destination (download);
-
+#ifdef HAVE_WEBKIT2
+  /* TODO: Downloads */
+#else
   webkit_download_start (priv->download);
+#endif
 }
 
 /**
@@ -636,7 +647,9 @@ ephy_download_do_download_action (EphyDownload *download,
     const char *destination_uri;
     EphyDownloadPrivate *priv;
     gboolean ret = FALSE;
-
+#ifdef HAVE_WEBKIT2
+    /* TODO: Downloads */
+#else
     priv = download->priv;
 
     destination_uri = webkit_download_get_destination_uri (priv->download);
@@ -659,13 +672,17 @@ ephy_download_do_download_action (EphyDownload *download,
         LOG ("ephy_download_do_download_action: none");
         ret = TRUE;
         break;
+      case EPHY_DOWNLOAD_ACTION_DO_NOTHING:
+        LOG ("ephy_download_do_download_action: nothing");
+        ret = TRUE;
+        break;
       default:
         LOG ("ephy_download_do_download_action: unhandled action");
         ret = FALSE;
         break;
     }
     g_object_unref (destination);
-
+#endif
     return ret;
 }
 
@@ -883,6 +900,9 @@ ephy_download_init (EphyDownload *download)
   download->priv->widget = NULL;
 }
 
+#ifdef HAVE_WEBKIT2
+/* TODO: Downloads */
+#else
 static void
 download_status_changed_cb (GObject *object,
                             GParamSpec *pspec,
@@ -898,10 +918,11 @@ download_status_changed_cb (GObject *object,
   if (status == WEBKIT_DOWNLOAD_STATUS_FINISHED) {
     g_signal_emit_by_name (download, "completed");
 
-    if (g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_AUTO_DOWNLOADS)) {
+    if (g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_AUTO_DOWNLOADS) &&
+        priv->action == EPHY_DOWNLOAD_ACTION_NONE) {
       ephy_download_do_download_action (download, EPHY_DOWNLOAD_ACTION_AUTO);
     } else {
-      ephy_download_do_download_action (download, EPHY_DOWNLOAD_ACTION_NONE);
+      ephy_download_do_download_action (download, priv->action);
     }
 
     ephy_embed_shell_remove_download (embed_shell, download);
@@ -930,6 +951,7 @@ download_error_cb (WebKitDownload *download,
 
   return ret;
 }
+#endif
 
 /**
  * ephy_download_new:
@@ -946,35 +968,6 @@ ephy_download_new (void)
   return g_object_new (EPHY_TYPE_DOWNLOAD, NULL);
 }
 
-static EphyDownload *
-_ephy_download_new (WebKitDownload *webkit_download, const char *uri)
-{
-  EphyDownload *ephy_download;
-  ephy_download = ephy_download_new ();
-
-  if (webkit_download == NULL) {
-    WebKitNetworkRequest *request;
-
-    request = webkit_network_request_new (uri);
-    webkit_download = webkit_download_new (request);
-
-    g_return_val_if_fail (webkit_download != NULL, NULL);
-    g_object_unref (request);
-  }
-
-  g_signal_connect (webkit_download, "notify::status",
-                    G_CALLBACK (download_status_changed_cb),
-                    ephy_download);
-  g_signal_connect (webkit_download, "error",
-                    G_CALLBACK (download_error_cb),
-                    ephy_download);
-
-  ephy_download->priv->download = g_object_ref (webkit_download);
-  ephy_download->priv->source = g_strdup (webkit_download_get_uri (webkit_download));
-
-  return ephy_download;
-}
-
 /**
  * ephy_download_new_for_download:
  * @download: a #WebKitDownload to wrap
@@ -986,9 +979,24 @@ _ephy_download_new (WebKitDownload *webkit_download, const char *uri)
 EphyDownload *
 ephy_download_new_for_download (WebKitDownload *download)
 {
-  g_return_val_if_fail (WEBKIT_IS_DOWNLOAD (download), NULL);
+  EphyDownload *ephy_download;
+  ephy_download = ephy_download_new ();
 
-  return _ephy_download_new (download, NULL);
+#ifdef HAVE_WEBKIT2
+  /* TODO: Downloads */
+#else
+  g_signal_connect (download, "notify::status",
+                    G_CALLBACK (download_status_changed_cb),
+                    ephy_download);
+  g_signal_connect (download, "error",
+                    G_CALLBACK (download_error_cb),
+                    ephy_download);
+
+  ephy_download->priv->download = g_object_ref (download);
+  ephy_download->priv->source = g_strdup (webkit_download_get_uri (download));
+#endif
+
+  return ephy_download;
 }
 
 /**
@@ -1002,7 +1010,25 @@ ephy_download_new_for_download (WebKitDownload *download)
 EphyDownload *
 ephy_download_new_for_uri (const char *uri)
 {
+  EphyDownload *ephy_download;
+  WebKitDownload *download;
+#ifdef HAVE_WEBKIT2
+  /* TODO: Downloads */
+  download = NULL;
+#else
+  WebKitNetworkRequest *request;
+
   g_return_val_if_fail (uri != NULL, NULL);
 
-  return _ephy_download_new (NULL, uri);
+  request = webkit_network_request_new (uri);
+  download = webkit_download_new (request);
+
+  g_return_val_if_fail (download != NULL, NULL);
+  g_object_unref (request);
+#endif
+
+  ephy_download = ephy_download_new_for_download (download);
+  g_object_unref (download);
+
+  return ephy_download;
 }
