@@ -29,7 +29,12 @@
 #include "ephy-settings.h"
 
 #include <glib.h>
+#include <math.h>
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
+#endif
 
 typedef struct
 {
@@ -39,7 +44,15 @@ typedef struct
   void (*callback) (GSettings *settings, char *key, gpointer data);
 } PrefData;
 
+#ifdef HAVE_WEBKIT2
+#define ENABLE_SCRIPTS_SETTING "enable-javascript"
+#define DEFAULT_ENCODING_SETTING "default-charset"
+static WebKitSettings *webkit_settings = NULL;
+#else
+#define ENABLE_SCRIPTS_SETTING "enable-scripts"
+#define DEFAULT_ENCODING_SETTING "default-encoding"
 static WebKitWebSettings *webkit_settings = NULL;
+#endif
 
 static void
 webkit_pref_callback_user_stylesheet (GSettings *settings,
@@ -60,8 +73,11 @@ webkit_pref_callback_user_stylesheet (GSettings *settings,
                        G_DIR_SEPARATOR_S,
                        USER_STYLESHEET_FILENAME,
                        NULL);
-
+#ifdef HAVE_WEBKIT2
+  /* TODO: user-stylesheet-uri setting */
+#else
   g_object_set (webkit_settings, webkit_pref, uri, NULL);
+#endif
   g_free (uri);
 }
 
@@ -106,6 +122,10 @@ webkit_pref_get_internal_user_agent (void)
 
   g_key_file_free (branding_keyfile);
 
+#ifdef HAVE_WEBKIT2
+  /* TODO: User agent */
+  return g_strdup ("");
+#else
   g_object_get (webkit_settings, "user-agent", &webkit_user_agent, NULL);
 
   user_agent = g_strconcat (webkit_user_agent, " ",
@@ -118,6 +138,7 @@ webkit_pref_get_internal_user_agent (void)
   g_free (webkit_user_agent);
 
   return user_agent;
+#endif
 }
 
 static void
@@ -125,6 +146,9 @@ webkit_pref_callback_user_agent (GSettings *settings,
                                  char *key,
                                  gpointer data)
 {
+#ifdef HAVE_WEBKIT2
+  /* TODO: User agent */
+#else
   char *value = NULL;
   static char *internal_user_agent = NULL;
   char *webkit_pref = data;
@@ -140,6 +164,49 @@ webkit_pref_callback_user_agent (GSettings *settings,
     g_object_set (webkit_settings, webkit_pref, value, NULL);
 
   g_free (value);
+#endif
+}
+
+#ifdef HAVE_WEBKIT2
+/* This doesn't contain WebKit2 specific API, but it's only used inside
+ * HAVE_WEBKIT2 blocks, so it gives a compile warning when building
+ * with WebKit1.
+ */
+static gdouble
+get_screen_dpi (GdkScreen *screen)
+{
+  gdouble dpi;
+  gdouble dp, di;
+
+  dpi = gdk_screen_get_resolution (screen);
+  if (dpi != -1)
+    return dpi;
+
+  dp = hypot (gdk_screen_get_width (screen), gdk_screen_get_height (screen));
+  di = hypot (gdk_screen_get_width_mm (screen), gdk_screen_get_height_mm (screen)) / 25.4;
+
+  return dp / di;
+}
+#endif
+
+static guint
+normalize_font_size (gdouble font_size)
+{
+#ifdef HAVE_WEBKIT2
+  /* WebKit2 uses font sizes in pixels. */
+  GdkScreen *screen;
+  gdouble dpi;
+
+  /* FIXME: We should use the view screen instead of the detault one
+   * but we don't have access to the view here.
+   */
+  screen = gdk_screen_get_default ();
+  dpi = screen ? get_screen_dpi (screen) : 96;
+
+  return font_size / 72.0 * dpi;
+#else
+  return font_size;
+#endif
 }
 
 static void
@@ -176,7 +243,7 @@ webkit_pref_callback_font_size (GSettings *settings,
     pango_font_description_free (desc);
   }
 
-  g_object_set (webkit_settings, webkit_pref, size, NULL);
+  g_object_set (webkit_settings, webkit_pref, normalize_font_size (size), NULL);
   g_free (value);
 }
 
@@ -266,6 +333,9 @@ webkit_pref_callback_accept_languages (GSettings *settings,
                                        char *key,
                                        gpointer data)
 {
+#ifdef HAVE_WEBKIT2
+  /* TODO: Languages */
+#else
   SoupSession *session;
   GArray *array;
   char **languages;
@@ -299,6 +369,7 @@ webkit_pref_callback_accept_languages (GSettings *settings,
   g_strfreev (languages);
   g_free (langs_str);
   g_array_free (array, TRUE);
+#endif
 }
 
 void
@@ -329,6 +400,9 @@ webkit_pref_callback_cookie_accept_policy (GSettings *settings,
                                            char *key,
                                            gpointer data)
 {
+#ifdef HAVE_WEBKIT2
+  /* TODO: Cookies */
+#else
   SoupSession *session;
   char *value = NULL;
 
@@ -348,6 +422,7 @@ webkit_pref_callback_cookie_accept_policy (GSettings *settings,
   }
 
   g_free (value);
+#endif
 }
 
 static void
@@ -362,8 +437,8 @@ webkit_pref_callback_gnome_fonts (GSettings *ephy_settings,
                   "default-font-family", "serif",
                   "sans-serif-font-family", "sans-serif",
                   "monospace-font-family", "monospace",
-                  "default-font-size", 12,
-                  "default-monospace-font-size", 10,
+                  "default-font-size", normalize_font_size (12),
+                  "default-monospace-font-size", normalize_font_size (10),
                   NULL);
   } else {
     /* Sync with Epiphany values */
@@ -385,6 +460,24 @@ webkit_pref_callback_gnome_fonts (GSettings *ephy_settings,
   }
 }
 
+static void
+replace_system_language_with_concrete_language_string (char **languages)
+{
+  int i;
+
+  for (i = 0; i < g_strv_length (languages); i++) {
+    if (g_str_equal (languages[i], "system")) {
+      char **sys_langs;
+
+      g_free (languages[i]);
+      sys_langs = ephy_langs_get_languages ();
+      languages[i] = g_strjoinv (", ", sys_langs);
+
+      g_strfreev (sys_langs);
+    }
+  }
+}
+
 static void
 webkit_pref_callback_enable_spell_checking (GSettings *settings,
                                             char *key,
@@ -398,13 +491,18 @@ webkit_pref_callback_enable_spell_checking (GSettings *settings,
 
   if (value) {
     languages = g_settings_get_strv (settings, EPHY_PREFS_WEB_LANGUAGE);
+    replace_system_language_with_concrete_language_string (languages);
     langs = g_strjoinv (",", languages);
     g_strdelimit (langs, "-", '_');
     g_strfreev (languages);
   }
 
+#ifdef HAVE_WEBKIT2
+  /* TODO: Spell checking */
+#else
   g_object_set (webkit_settings, "enable-spell-checking", value, NULL);
   g_object_set (webkit_settings, "spell-checking-languages", langs, NULL);
+#endif
 
   g_free (langs);
 }
@@ -465,8 +563,13 @@ static const PrefData webkit_pref_entries[] =
       webkit_pref_callback_cookie_accept_policy },
   };
 
+#ifdef HAVE_WEBKIT2
+static void
+ephy_embed_prefs_apply (EphyEmbed *embed, WebKitSettings *settings)
+#else
 static void
 ephy_embed_prefs_apply (EphyEmbed *embed, WebKitWebSettings *settings)
+#endif
 {
   webkit_web_view_set_settings (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed),
                                 settings);
@@ -476,7 +579,12 @@ void
 ephy_embed_prefs_init (void)
 {
   int i;
-
+#ifdef HAVE_WEBKIT2
+  /* TODO: site-specific-quirks and page-cache settings */
+  webkit_settings = webkit_settings_new_with_settings ("enable-developer-extras", TRUE,
+                                                       "enable-fullscreen", TRUE,
+                                                       NULL);
+#else
   webkit_settings = webkit_web_settings_new ();
 
   /* Hardcoded settings */
@@ -487,6 +595,7 @@ ephy_embed_prefs_init (void)
                 "enable-developer-extras", TRUE,
                 "enable-fullscreen", TRUE,
                 NULL);
+#endif
 
   for (i = 0; i < G_N_ELEMENTS (webkit_pref_entries); i++) {
     GSettings *settings;
@@ -507,7 +616,7 @@ ephy_embed_prefs_init (void)
 
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_ENABLE_JAVASCRIPT,
-                   webkit_settings, "enable-scripts",
+                   webkit_settings, ENABLE_SCRIPTS_SETTING,
                    G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_MAIN,
                    EPHY_PREFS_ENABLE_CARET_BROWSING,
@@ -527,7 +636,11 @@ ephy_embed_prefs_init (void)
                    G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_DEFAULT_ENCODING,
-                   webkit_settings, "default-encoding",
+                   webkit_settings, DEFAULT_ENCODING_SETTING,
+                   G_SETTINGS_BIND_GET);
+  g_settings_bind (EPHY_SETTINGS_WEB,
+                   EPHY_PREFS_WEB_ENABLE_WEBGL,
+                   webkit_settings, "enable-webgl",
                    G_SETTINGS_BIND_GET);
 }
 
