@@ -36,8 +36,13 @@
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-request-about.h"
+#include "ephy-about-handler.h"
 
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
+#endif
 #include <glib/gi18n.h>
 #include <libsoup/soup-gnome.h>
 #include <libsoup/soup-cache.h>
@@ -333,6 +338,62 @@ cache_size_cb (GSettings *settings,
   soup_cache_set_max_size (single->priv->cache, new_cache_size * 1024 * 1024 /* in bytes */);
 }
 
+#ifdef HAVE_WEBKIT2
+static void
+complete_about_request_for_contents (WebKitURISchemeRequest *request,
+                                     gchar *data,
+                                     gsize data_length)
+{
+  GInputStream *stream;
+
+  stream = g_memory_input_stream_new_from_data (data, data_length, g_free);
+  webkit_uri_scheme_request_finish (request, stream, data_length, "text/html");
+  g_object_unref (stream);
+}
+
+static void
+get_plugins_cb (WebKitWebContext *web_context,
+                GAsyncResult *result,
+                WebKitURISchemeRequest *request)
+{
+  GList *plugins;
+  GString *data_str;
+  gsize data_length;
+
+  data_str = g_string_new("<html>");
+  plugins = webkit_web_context_get_plugins_finish (web_context, result, NULL);
+  _ephy_about_handler_handle_plugins (data_str, plugins);
+  g_string_append (data_str, "</html>");
+
+  data_length = data_str->len;
+  complete_about_request_for_contents (request, g_string_free (data_str, FALSE), data_length);
+  g_object_unref (request);
+}
+
+static void
+about_request_cb (WebKitURISchemeRequest *request,
+                  gpointer user_data)
+{
+  const gchar *path;
+
+  path = webkit_uri_scheme_request_get_path (request);
+  if (!g_strcmp0 (path, "plugins")) {
+    /* Plugins API is async in WebKit2 */
+    webkit_web_context_get_plugins (webkit_web_context_get_default (),
+                                    NULL,
+                                    (GAsyncReadyCallback) get_plugins_cb,
+                                    g_object_ref (request));
+  } else {
+    GString *contents;
+    gsize data_length;
+
+    contents = ephy_about_handler_handle (path);
+    data_length = contents->len;
+    complete_about_request_for_contents (request, g_string_free (contents, FALSE), data_length);
+  }
+}
+#endif
+
 /**
  * ephy_embed_single_initialize:
  * @single: the #EphyEmbedSingle
@@ -343,6 +404,13 @@ cache_size_cb (GSettings *settings,
 gboolean
 ephy_embed_single_initialize (EphyEmbedSingle *single)
 {
+#ifdef HAVE_WEBKIT2
+  /* TODO: Network features */
+  webkit_web_context_register_uri_scheme (webkit_web_context_get_default (),
+                                          EPHY_ABOUT_SCHEME,
+                                          about_request_cb,
+                                          NULL);
+#else
   SoupSession *session;
   SoupCookieJar *jar;
   char *filename;
@@ -413,7 +481,7 @@ ephy_embed_single_initialize (EphyEmbedSingle *single)
   favicon_db_path = g_build_filename (g_get_user_data_dir (), g_get_prgname (), NULL);
   webkit_favicon_database_set_path (webkit_get_favicon_database (), favicon_db_path);
   g_free (favicon_db_path);
-
+#endif
   return TRUE;
 }
 
