@@ -26,13 +26,14 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
+#include <libsoup/soup.h>
 
 #include "ephy-string.h"
 #include "ephy-embed-utils.h"
-#include "ephy-request-about.h"
+#include "ephy-about-handler.h"
 
-char*
-ephy_embed_utils_link_message_parse (char *message)
+char *
+ephy_embed_utils_link_message_parse (const char *message)
 {
 	
 	char *status_message;
