@@ -29,29 +29,29 @@
 #include "ephy-adblock-manager.h"
 #include "ephy-debug.h"
 #include "ephy-download.h"
-#include "ephy-embed-event.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-shell.h"
-#include "ephy-embed-single.h"
-#include "ephy-embed-utils.h"
-#include "ephy-file-helpers.h"
-#include "ephy-history.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
-#include "ephy-string.h"
 #include "ephy-web-view.h"
 #include "nautilus-floating-bar.h"
 
-#include <errno.h>
 #include <glib/gi18n.h>
-#include <string.h>
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
+#endif
 
 static void     ephy_embed_constructed      (GObject *object);
+#ifdef HAVE_WEBKIT2
+/* TODO: Inspector */
+#else
 static gboolean ephy_embed_inspect_show_cb  (WebKitWebInspector *inspector,
                                              EphyEmbed *embed);
 static gboolean ephy_embed_inspect_close_cb (WebKitWebInspector *inspector,
                                              EphyEmbed *embed);
+#endif
 
 #define EPHY_EMBED_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED, EphyEmbedPrivate))
 
@@ -66,18 +66,20 @@ typedef struct {
 struct _EphyEmbedPrivate
 {
   GtkBox *top_widgets_vbox;
+#ifndef HAVE_WEBKIT2
   GtkScrolledWindow *scrolled_window;
+#endif
   GtkPaned *paned;
   WebKitWebView *web_view;
-  EphyHistory *history;
   GtkWidget *inspector_window;
   GtkWidget *inspector_web_view;
   GtkWidget *inspector_scrolled_window;
   gboolean inspector_attached;
-  guint is_setting_zoom : 1;
   GSList *destroy_on_transition_list;
   GtkWidget *floating_bar;
   GtkWidget *progress;
+  GtkWidget *fullscreen_message_label;
+  char *fullscreen_string;
 
   GSList *messages;
   GSList *keys;
@@ -88,6 +90,8 @@ struct _EphyEmbedPrivate
   guint tab_message_id;
   guint pop_statusbar_later_source_id;
 
+  guint fullscreen_message_id;
+
   guint clear_progress_source_id;
 
   gulong status_handler_id;
@@ -226,43 +230,9 @@ ephy_embed_statusbar_pop (EphyEmbed *embed, guint context_id)
   ephy_embed_statusbar_update (embed, msg ? msg->text : NULL);
 }
 
-static void
-restore_zoom_level (EphyEmbed *embed,
-                    const char *address)
-{
-  EphyEmbedPrivate *priv = embed->priv;
-
-  /* restore zoom level */
-  if (ephy_embed_utils_address_has_web_scheme (address)) {
-    EphyHistory *history;
-    EphyNode *host;
-    WebKitWebView *web_view;
-    GValue value = { 0, };
-    float zoom = 1.0, current_zoom;
-
-    history = EPHY_HISTORY
-              (ephy_embed_shell_get_global_history (embed_shell));
-    host = ephy_history_get_host (history, address);
-
-    if (host != NULL && ephy_node_get_property
-        (host, EPHY_NODE_HOST_PROP_ZOOM, &value)) {
-      zoom = g_value_get_float (&value);
-      g_value_unset (&value);
-    }
-
-    web_view = priv->web_view;
-
-    g_object_get (web_view, "zoom-level", &current_zoom,
-                  NULL);
-
-    if (zoom != current_zoom) {
-      priv->is_setting_zoom = TRUE;
-      g_object_set (web_view, "zoom-level", zoom, NULL);
-      priv->is_setting_zoom = FALSE;
-    }
-  }
-}
-
+#ifdef HAVE_WEBKIT2
+/* TODO: WebKitWebResource::send-request */
+#else
 static void
 resource_request_starting_cb (WebKitWebView *web_view,
                               WebKitWebFrame *web_frame,
@@ -285,6 +255,7 @@ resource_request_starting_cb (WebKitWebView *web_view,
     webkit_network_request_set_uri (request, "about:blank");
   }
 }
+#endif
 
 static void
 ephy_embed_destroy_top_widgets (EphyEmbed *embed)
@@ -305,70 +276,27 @@ remove_from_destroy_list_cb (GtkWidget *widget, EphyEmbed *embed)
   embed->priv->destroy_on_transition_list = list;
 }
 
+#ifdef HAVE_WEBKIT2
 static void
-load_status_changed_cb (WebKitWebView *view,
+load_changed_cb (WebKitWebView *web_view,
+                 WebKitLoadEvent load_event,
+                 EphyEmbed *embed)
+{
+  if (load_event == WEBKIT_LOAD_COMMITTED)
+    ephy_embed_destroy_top_widgets (embed);
+}
+#else
+static void
+load_status_changed_cb (WebKitWebView *web_view,
                         GParamSpec *spec,
                         EphyEmbed *embed)
 {
-  WebKitLoadStatus status = webkit_web_view_get_load_status (view);
-
-  if (status == WEBKIT_LOAD_COMMITTED) {
-    const gchar* uri;
-
-    uri = webkit_web_view_get_uri (view);
+  WebKitLoadStatus status = webkit_web_view_get_load_status (web_view);
 
+  if (status == WEBKIT_LOAD_COMMITTED)
     ephy_embed_destroy_top_widgets (embed);
-
-    restore_zoom_level (embed, uri);
-
-    /* FIXME: we are not identifying redirects at the moment */
-    ephy_history_add_page (embed->priv->history,
-                           uri,
-                           FALSE,
-                           FALSE);
-  }
-}
-
-static void
-zoom_changed_cb (WebKitWebView *web_view,
-                 GParamSpec *pspec,
-                 EphyEmbed  *embed)
-{
-  char *address;
-  float zoom;
-
-  g_object_get (web_view,
-                "zoom-level", &zoom,
-                NULL);
-
-  if (EPHY_EMBED (embed)->priv->is_setting_zoom) {
-    return;
-  }
-
-  address = ephy_web_view_get_location (EPHY_WEB_VIEW (web_view), TRUE);
-  if (ephy_embed_utils_address_has_web_scheme (address)) {
-    EphyHistory *history;
-    EphyNode *host;
-    history = EPHY_HISTORY
-      (ephy_embed_shell_get_global_history (embed_shell));
-    host = ephy_history_get_host (history, address);
-
-    if (host != NULL) {
-      ephy_node_set_property_float (host,
-                                    EPHY_NODE_HOST_PROP_ZOOM,
-                                    zoom);
-    }
-  }
-
-  g_free (address);
-}
-
-static void
-ephy_embed_history_cleared_cb (EphyHistory *history,
-                               EphyEmbed *embed)
-{
-  ephy_web_view_clear_history (EPHY_WEB_VIEW (embed->priv->web_view));
 }
+#endif
 
 static void
 ephy_embed_grab_focus (GtkWidget *widget)
@@ -381,6 +309,38 @@ ephy_embed_grab_focus (GtkWidget *widget)
     gtk_widget_grab_focus (child);
 }
 
+
+static gboolean
+fullscreen_message_label_hide (EphyEmbed *embed)
+{
+  if (embed->priv->fullscreen_message_id) {
+    gtk_widget_hide (embed->priv->fullscreen_message_label);
+    g_source_remove (embed->priv->fullscreen_message_id);
+    embed->priv->fullscreen_message_id = 0;
+  }
+
+  return FALSE;
+}
+
+void
+ephy_embed_entering_fullscreen (EphyEmbed *embed)
+{
+  gtk_widget_show (embed->priv->fullscreen_message_label);
+
+  if (embed->priv->fullscreen_message_id)
+    g_source_remove (embed->priv->fullscreen_message_id);
+
+  embed->priv->fullscreen_message_id = g_timeout_add_seconds (5,
+                                                              (GSourceFunc)fullscreen_message_label_hide,
+                                                              embed);
+}
+
+void
+ephy_embed_leaving_fullscreen (EphyEmbed *embed)
+{
+  fullscreen_message_label_hide (embed);
+}
+
 static void
 ephy_embed_dispose (GObject *object)
 {
@@ -388,6 +348,9 @@ ephy_embed_dispose (GObject *object)
   EphyEmbedPrivate *priv = embed->priv;
 
   if (priv->inspector_window) {
+#ifdef HAVE_WEBKIT2
+    /* TODO: Inspector */
+#else
     WebKitWebInspector *inspector;
 
     inspector = webkit_web_view_get_inspector (priv->web_view);
@@ -399,6 +362,7 @@ ephy_embed_dispose (GObject *object)
     g_signal_handlers_disconnect_by_func (inspector,
                                           ephy_embed_inspect_close_cb,
                                           priv->inspector_window);
+#endif
 
     gtk_widget_destroy (GTK_WIDGET (priv->inspector_window));
     priv->inspector_window = NULL;
@@ -426,6 +390,11 @@ ephy_embed_dispose (GObject *object)
     priv->progress_update_handler_id = 0;
   }
 
+  if (priv->fullscreen_message_id) {
+    g_source_remove (priv->fullscreen_message_id);
+    priv->fullscreen_message_id = 0;
+  }
+
   G_OBJECT_CLASS (ephy_embed_parent_class)->dispose (object);
 }
 
@@ -443,10 +412,6 @@ ephy_embed_finalize (GObject *object)
   }
   g_slist_free (priv->destroy_on_transition_list);
 
-  g_signal_handlers_disconnect_by_func (priv->history,
-                                        ephy_embed_history_cleared_cb,
-                                        embed);
-
   for (list = priv->messages; list; list = list->next) {
     EphyEmbedStatusbarMsg *msg;
 
@@ -464,6 +429,8 @@ ephy_embed_finalize (GObject *object)
   g_slist_free (priv->keys);
   priv->keys = NULL;
 
+  g_free (embed->priv->fullscreen_string);
+
   G_OBJECT_CLASS (ephy_embed_parent_class)->finalize (object);
 }
 
@@ -481,6 +448,9 @@ ephy_embed_class_init (EphyEmbedClass *klass)
   g_type_class_add_private (G_OBJECT_CLASS (klass), sizeof(EphyEmbedPrivate));
 }
 
+#ifdef HAVE_WEBKIT2
+/* TODO: Inspector */
+#else
 static WebKitWebView *
 ephy_embed_inspect_web_view_cb (WebKitWebInspector *inspector,
                                 WebKitWebView *web_view,
@@ -569,6 +539,7 @@ ephy_embed_inspect_close_cb (WebKitWebInspector *inspector,
 
   return TRUE;
 }
+#endif
 
 void
 ephy_embed_auto_download_url (EphyEmbed *embed, const char *url)
@@ -580,6 +551,9 @@ ephy_embed_auto_download_url (EphyEmbed *embed, const char *url)
   ephy_download_set_action (download, EPHY_DOWNLOAD_ACTION_OPEN);
 }
 
+#ifdef HAVE_WEBKIT2
+/* TODO: Downloads */
+#else
 static gboolean
 download_requested_cb (WebKitWebView *web_view,
                        WebKitDownload *download,
@@ -601,6 +575,41 @@ download_requested_cb (WebKitWebView *web_view,
 
   return TRUE;
 }
+#endif
+
+static void
+ephy_embed_set_fullscreen_message (EphyEmbed *embed,
+                                   gboolean is_html5_fullscreen)
+{
+  char *message;
+
+  if (G_UNLIKELY (embed->priv->fullscreen_string == NULL))
+    embed->priv->fullscreen_string = g_strdup (_("Press %s to exit fullscreen"));
+
+  /* Translators: 'ESC' and 'F11' are keyboard keys. */
+  message = g_strdup_printf (embed->priv->fullscreen_string, is_html5_fullscreen ? _("ESC") : _("F11"));
+  gtk_label_set_text (GTK_LABEL (embed->priv->fullscreen_message_label),
+                      message);
+  g_free (message);
+}
+
+static gboolean
+entering_fullscreen_cb (WebKitWebView *web_view,
+                        GObject *element,
+                        EphyEmbed *embed)
+{
+  ephy_embed_set_fullscreen_message (embed, TRUE);
+  return FALSE;
+}
+
+static gboolean
+leaving_fullscreen_cb (WebKitWebView *web_view,
+                       GObject *element,
+                       EphyEmbed *embed)
+{
+  ephy_embed_set_fullscreen_message (embed, FALSE);
+  return FALSE;
+}
 
 static gboolean
 pop_statusbar_later_cb (gpointer data)
@@ -638,6 +647,29 @@ status_message_notify_cb (EphyWebView *view, GParamSpec *pspec, EphyEmbed *embed
   }
 }
 
+#ifdef HAVE_WEBKIT2
+static void
+window_geometry_changed (WebKitWindowProperties *properties, GParamSpec *pspec, EphyEmbed *embed)
+{
+  GtkWidget *window;
+  gboolean is_popup;
+  GdkRectangle geometry;
+
+  window = gtk_widget_get_toplevel (GTK_WIDGET (embed));
+  if (!window || !gtk_widget_is_toplevel (window))
+    return;
+
+  g_object_get (window, "is-popup", &is_popup, NULL);
+  if (!is_popup)
+    return;
+
+  webkit_window_properties_get_geometry (properties, &geometry);
+  if (geometry.x >= 0 && geometry.y >= 0)
+    gtk_window_move (GTK_WINDOW (window), geometry.x, geometry.y);
+  if (geometry.width > 0 && geometry.height > 0)
+    gtk_window_resize (GTK_WINDOW (window), geometry.width, geometry.height);
+}
+#else
 static void
 window_resize_requested (WebKitWebWindowFeatures *features, GParamSpec *pspec, EphyEmbed *embed)
 {
@@ -666,6 +698,7 @@ window_resize_requested (WebKitWebWindowFeatures *features, GParamSpec *pspec, E
   g_object_get (features, "width", &width, "height", &height, NULL);
   gtk_window_resize (GTK_WINDOW (window), width, height);
 }
+#endif
 
 static gboolean
 clear_progress_cb (EphyEmbed *embed)
@@ -691,10 +724,14 @@ progress_update (EphyWebView *view, GParamSpec *pspec, EphyEmbed *embed)
   }
 
   uri = webkit_web_view_get_uri (priv->web_view);
-  if (!uri || strcmp (uri, "about:blank") == 0)
+  if (!uri || g_str_equal (uri, "about:blank"))
     return;
 
+#ifdef HAVE_WEBKIT2
+  progress = webkit_web_view_get_estimated_load_progress (priv->web_view);
+#else
   progress = webkit_web_view_get_progress (priv->web_view);
+#endif
   loading = ephy_web_view_is_loading (EPHY_WEB_VIEW (priv->web_view));
 
   if (progress == 1.0 || !loading)
@@ -713,22 +750,48 @@ ephy_embed_constructed (GObject *object)
 {
   EphyEmbed *embed = (EphyEmbed*)object;
   EphyEmbedPrivate *priv = embed->priv;
+#ifndef HAVE_WEBKIT2
   GtkWidget *scrolled_window;
+#endif
   GtkWidget *paned;
   WebKitWebView *web_view;
+#ifdef HAVE_WEBKIT2
+  WebKitWindowProperties *window_properties;
+#else
   WebKitWebWindowFeatures *window_features;
+#endif
+#ifndef HAVE_WEBKIT2
+  /* TODO: Inspector */
   WebKitWebInspector *inspector;
+#endif
   GtkWidget *overlay;
 
   /* Skeleton */
   web_view = WEBKIT_WEB_VIEW (ephy_web_view_new ());
+#ifndef HAVE_WEBKIT2
   scrolled_window = GTK_WIDGET (priv->scrolled_window);
+#endif
   overlay = gtk_overlay_new ();
+  gtk_style_context_add_class (gtk_widget_get_style_context (overlay),
+                               GTK_STYLE_CLASS_OSD);
+
   gtk_widget_add_events (overlay, 
                          GDK_ENTER_NOTIFY_MASK |
                          GDK_LEAVE_NOTIFY_MASK);
-  gtk_widget_set_name (overlay, "ephy-overlay");
+#ifdef HAVE_WEBKIT2
+  gtk_container_add (GTK_CONTAINER (overlay), GTK_WIDGET (web_view));
+#else
   gtk_container_add (GTK_CONTAINER (overlay), scrolled_window);
+#endif
+
+  /* Floating message popup for fullscreen mode. */
+  priv->fullscreen_message_label = gtk_label_new (NULL);
+  gtk_widget_set_name (priv->fullscreen_message_label, "fullscreen-popup");
+  gtk_widget_set_halign (priv->fullscreen_message_label, GTK_ALIGN_CENTER);
+  gtk_widget_set_valign (priv->fullscreen_message_label, GTK_ALIGN_CENTER);
+  gtk_widget_set_no_show_all (priv->fullscreen_message_label, TRUE);
+  gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->fullscreen_message_label);
+  ephy_embed_set_fullscreen_message (embed, FALSE);
 
   /* statusbar is hidden by default */
   priv->floating_bar = nautilus_floating_bar_new (NULL, FALSE);
@@ -739,7 +802,6 @@ ephy_embed_constructed (GObject *object)
   gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->floating_bar);
 
   priv->progress = gtk_progress_bar_new ();
-  gtk_widget_set_name (priv->progress, "ephy-progress-bar");
   gtk_widget_set_halign (priv->progress, GTK_ALIGN_FILL);
   gtk_widget_set_valign (priv->progress, GTK_ALIGN_START);
   gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->progress);
@@ -747,11 +809,18 @@ ephy_embed_constructed (GObject *object)
   paned = GTK_WIDGET (priv->paned);
 
   priv->web_view = web_view;
+#ifdef HAVE_WEBKIT2
+  priv->progress_update_handler_id = g_signal_connect (web_view, "notify::estimated-load-progress",
+                                                       G_CALLBACK (progress_update), object);
+#else
   priv->progress_update_handler_id =  g_signal_connect (web_view, "notify::progress",
                                                         G_CALLBACK (progress_update), object);
+#endif
 
+#ifndef HAVE_WEBKIT2
   gtk_container_add (GTK_CONTAINER (scrolled_window),
                      GTK_WIDGET (web_view));
+#endif
   gtk_paned_pack1 (GTK_PANED (paned), GTK_WIDGET (overlay),
                    TRUE, FALSE);
 
@@ -764,17 +833,31 @@ ephy_embed_constructed (GObject *object)
   gtk_widget_show (GTK_WIDGET (web_view));
   gtk_widget_show_all (paned);
 
+#ifdef HAVE_WEBKIT2
+  /* TODO: WebKitWebResource::send-request, Downloads */
+  g_signal_connect (web_view, "load-changed",
+                    G_CALLBACK (load_changed_cb),
+                    embed);
+#else
   g_object_connect (web_view,
                     "signal::notify::load-status", G_CALLBACK (load_status_changed_cb), embed,
                     "signal::resource-request-starting", G_CALLBACK (resource_request_starting_cb), embed,
                     "signal::download-requested", G_CALLBACK (download_requested_cb), embed,
-                    "signal::notify::zoom-level", G_CALLBACK (zoom_changed_cb), embed,
+                    "signal::entering-fullscreen", G_CALLBACK (entering_fullscreen_cb), embed,
+                    "signal::leaving-fullscreen", G_CALLBACK (leaving_fullscreen_cb), embed,
                     NULL);
+#endif
 
   priv->status_handler_id = g_signal_connect (web_view, "notify::status-message",
                                               G_CALLBACK (status_message_notify_cb),
                                               embed);
-
+#ifdef HAVE_WEBKIT2
+  /* Window properties */
+  window_properties = webkit_web_view_get_window_properties (web_view);
+  g_signal_connect (window_properties, "notify::geometry",
+                    G_CALLBACK (window_geometry_changed),
+                    embed);
+#else
   /* Window features */
   window_features = webkit_web_view_get_window_features (web_view);
   g_object_connect (window_features,
@@ -783,9 +866,13 @@ ephy_embed_constructed (GObject *object)
                     "signal::notify::width", G_CALLBACK (window_resize_requested), embed,
                     "signal::notify::height", G_CALLBACK (window_resize_requested), embed,
                     NULL);
+#endif
 
   /* The inspector */
   priv->inspector_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+#ifdef HAVE_WEBKIT2
+  /* TODO: Inspector */
+#else
   inspector = webkit_web_view_get_inspector (web_view);
 
   priv->inspector_scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -815,14 +902,9 @@ ephy_embed_constructed (GObject *object)
                     "signal::detach-window", G_CALLBACK (ephy_embed_detach_inspector_cb),
                     embed,
                     NULL);
+#endif
 
   ephy_embed_prefs_add_embed (embed);
-
-  priv->history = EPHY_HISTORY (ephy_embed_shell_get_global_history (ephy_embed_shell_get_default ()));
-
-  g_signal_connect (priv->history,
-                    "cleared", G_CALLBACK (ephy_embed_history_cleared_cb),
-                    embed);
 }
 
 static void
@@ -833,15 +915,19 @@ ephy_embed_init (EphyEmbed *embed)
   gtk_orientable_set_orientation (GTK_ORIENTABLE (embed),
                                   GTK_ORIENTATION_VERTICAL);
 
+#ifndef HAVE_WEBKIT2
   embed->priv->scrolled_window = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
+#endif
   embed->priv->paned = GTK_PANED (gtk_paned_new (GTK_ORIENTATION_VERTICAL));
   embed->priv->top_widgets_vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 0));
   embed->priv->seq_context_id = 1;
   embed->priv->seq_message_id = 1;
   embed->priv->tab_message_id = ephy_embed_statusbar_get_context_id (embed, EPHY_EMBED_STATUSBAR_TAB_MESSAGE_CONTEXT_DESCRIPTION);
 
+#ifndef HAVE_WEBKIT2
   gtk_scrolled_window_set_policy (embed->priv->scrolled_window,
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+#endif
 }
 
 /**
@@ -892,7 +978,7 @@ ephy_embed_add_top_widget (EphyEmbed *embed, GtkWidget *widget, gboolean destroy
  * @widget: a #GtkWidget
  *
  * Removes an #GtkWidget from the top of the embed. The #GtkWidget
- * must be have been added using ephy_embed_add_widget(), and not
+ * must have been added using ephy_embed_add_top_widget(), and not
  * have been removed by other means. See gtk_container_remove() for
  * details.
  */
