@@ -1,5 +1,8 @@
+/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set sw=2 ts=2 sts=2 et: */
 /*
  *  Copyright © 2008 Gustavo Noronha Silva
+ *  Copyright © 2012 Igalia S.L.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +18,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
@@ -25,12 +27,12 @@
 #ifndef EPHY_WEB_VIEW_H
 #define EPHY_WEB_VIEW_H
 
-#include <glib-object.h>
-#include <glib.h>
 #include <gtk/gtk.h>
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
-
-#include "ephy-embed-event.h"
+#endif
 
 G_BEGIN_DECLS
 
@@ -47,40 +49,34 @@ typedef struct _EphyWebViewPrivate  EphyWebViewPrivate;
 
 typedef enum
 {
-	EPHY_WEB_VIEW_NAV_UP		= 1 << 0,
-	EPHY_WEB_VIEW_NAV_BACK		= 1 << 1,
-	EPHY_WEB_VIEW_NAV_FORWARD	= 1 << 2
+  EPHY_WEB_VIEW_NAV_BACK    = 1 << 0,
+  EPHY_WEB_VIEW_NAV_FORWARD = 1 << 1
 } EphyWebViewNavigationFlags;
 
 typedef enum
 {
-	EPHY_WEB_VIEW_CHROME_MENUBAR		= 1 << 0,
-	EPHY_WEB_VIEW_CHROME_TOOLBAR		= 1 << 1,
-	EPHY_WEB_VIEW_CHROME_STATUSBAR		= 1 << 2,
-	EPHY_WEB_VIEW_CHROME_BOOKMARKSBAR	= 1 << 3
+  EPHY_WEB_VIEW_CHROME_TOOLBAR    = 1 << 0,
+  EPHY_WEB_VIEW_CHROME_DOWNLOADS_BOX = 1 << 1
 } EphyWebViewChrome;
 
-#define EPHY_WEB_VIEW_CHROME_ALL (EPHY_WEB_VIEW_CHROME_MENUBAR |	\
-			       EPHY_WEB_VIEW_CHROME_TOOLBAR |	\
-			       EPHY_WEB_VIEW_CHROME_STATUSBAR |	\
-			       EPHY_WEB_VIEW_CHROME_BOOKMARKSBAR)
+#define EPHY_WEB_VIEW_CHROME_ALL (EPHY_WEB_VIEW_CHROME_TOOLBAR)
 
 typedef enum
 {
-	EPHY_WEB_VIEW_STATE_IS_UNKNOWN,
-	EPHY_WEB_VIEW_STATE_IS_INSECURE,
-	EPHY_WEB_VIEW_STATE_IS_BROKEN,
-	EPHY_WEB_VIEW_STATE_IS_SECURE_LOW,
-	EPHY_WEB_VIEW_STATE_IS_SECURE_MED,
-	EPHY_WEB_VIEW_STATE_IS_SECURE_HIGH
+  EPHY_WEB_VIEW_STATE_IS_UNKNOWN,
+  EPHY_WEB_VIEW_STATE_IS_INSECURE,
+  EPHY_WEB_VIEW_STATE_IS_BROKEN,
+  EPHY_WEB_VIEW_STATE_IS_SECURE_LOW,
+  EPHY_WEB_VIEW_STATE_IS_SECURE_MED,
+  EPHY_WEB_VIEW_STATE_IS_SECURE_HIGH
 } EphyWebViewSecurityLevel;
 
 typedef enum
 {
-	EPHY_WEB_VIEW_DOCUMENT_HTML,
-	EPHY_WEB_VIEW_DOCUMENT_XML,
-	EPHY_WEB_VIEW_DOCUMENT_IMAGE,
-	EPHY_WEB_VIEW_DOCUMENT_OTHER
+  EPHY_WEB_VIEW_DOCUMENT_HTML,
+  EPHY_WEB_VIEW_DOCUMENT_XML,
+  EPHY_WEB_VIEW_DOCUMENT_IMAGE,
+  EPHY_WEB_VIEW_DOCUMENT_OTHER
 } EphyWebViewDocumentType;
 
 typedef enum {
@@ -101,30 +97,29 @@ struct _EphyWebViewClass
   WebKitWebViewClass parent_class;
 
   /* Signals */
-  void	 (* feed_link)		(EphyWebView *view,
+  void   (* feed_link)          (EphyWebView *view,
                                  const char *type,
                                  const char *title,
                                  const char *address);
-  void	 (* search_link)	(EphyWebView *view,
+  void   (* search_link)        (EphyWebView *view,
                                  const char *type,
                                  const char *title,
                                  const char *address);
-  void	 (* popup_blocked)	(EphyWebView *view,
+  void   (* popup_blocked)      (EphyWebView *view,
                                  const char *address,
                                  const char *target,
                                  const char *features);
-  void	 (* content_blocked)	(EphyWebView *view,
+  void   (* content_blocked)    (EphyWebView *view,
                                  const char *uri);
-  gboolean (* modal_alert)	(EphyWebView *view);
-  void	 (* modal_alert_closed) (EphyWebView *view);
-  void	 (* new_window)		(EphyWebView *view,
+  gboolean (* modal_alert)      (EphyWebView *view);
+  void   (* modal_alert_closed) (EphyWebView *view);
+  void   (* new_window)         (EphyWebView *view,
                                  EphyWebView *new_view);
-  gboolean (* search_key_press)	(EphyWebView *view,
+  gboolean (* search_key_press) (EphyWebView *view,
                                  GdkEventKey *event);
-  gboolean (* close_request)	(EphyWebView *view);
-
-  void	 (* new_document_now)	(EphyWebView *view,
+  void   (* new_document_now)   (EphyWebView *view,
                                  const char *uri);
+  void   (* loading_homepage)   (EphyWebView *view);
 };
 
 GType                      ephy_web_view_get_type                 (void);
@@ -132,13 +127,17 @@ GType                      ephy_web_view_chrome_get_type          (void);
 GType                      ephy_web_view_security_level_get_type  (void);
 GtkWidget *                ephy_web_view_new                      (void);
 void                       ephy_web_view_load_request             (EphyWebView               *view,
+#ifdef HAVE_WEBKIT2
+                                                                   WebKitURIRequest          *request);
+#else
                                                                    WebKitNetworkRequest      *request);
+#endif
 void                       ephy_web_view_load_url                 (EphyWebView               *view,
                                                                    const char                *url);
 void                       ephy_web_view_copy_back_history        (EphyWebView               *source,
                                                                    EphyWebView               *dest);
-void                       ephy_web_view_clear_history            (EphyWebView               *view);
 gboolean                   ephy_web_view_is_loading               (EphyWebView               *view);
+gboolean                   ephy_web_view_load_failed              (EphyWebView               *view);
 const char *               ephy_web_view_get_loading_title        (EphyWebView               *view);
 GdkPixbuf *                ephy_web_view_get_icon                 (EphyWebView               *view);
 EphyWebViewDocumentType    ephy_web_view_get_document_type        (EphyWebView               *view);
@@ -147,7 +146,7 @@ const char *               ephy_web_view_get_status_message       (EphyWebView
 const char *               ephy_web_view_get_link_message         (EphyWebView               *view);
 gboolean                   ephy_web_view_get_visibility           (EphyWebView               *view);
 void                       ephy_web_view_set_link_message         (EphyWebView               *view,
-                                                                   char                      *link_message);
+                                                                   const char                *link_message);
 void                       ephy_web_view_set_security_level       (EphyWebView               *view,
                                                                    EphyWebViewSecurityLevel   level);
 void                       ephy_web_view_set_visibility           (EphyWebView               *view,
@@ -159,25 +158,20 @@ gboolean                   ephy_web_view_get_is_blank             (EphyWebView
 gboolean                   ephy_web_view_has_modified_forms       (EphyWebView               *view);
 char *                     ephy_web_view_get_location             (EphyWebView               *view,
                                                                    gboolean                   toplevel);
-void                       ephy_web_view_go_up                    (EphyWebView               *view);
 void                       ephy_web_view_get_security_level       (EphyWebView               *view,
                                                                    EphyWebViewSecurityLevel  *level,
                                                                    char                     **description);
-void                       ephy_web_view_show_page_certificate    (EphyWebView               *view);
 void                       ephy_web_view_print                    (EphyWebView               *view);
 void                       ephy_web_view_set_title                (EphyWebView               *view,
                                                                    const char                *view_title);
 const char *               ephy_web_view_get_title                (EphyWebView               *view);
-gboolean                   ephy_web_view_can_go_up                (EphyWebView               *view);
 const char *               ephy_web_view_get_address              (EphyWebView               *view);
 const char *               ephy_web_view_get_title_composite      (EphyWebView               *view);
 
-void                       ephy_web_view_load_error_page          (EphyWebView		     *view,
-                                                                   const char		     *uri,
-                                                                   EphyWebViewErrorPage	      page,
-                                                                   GError		     *error);
-
-
+void                       ephy_web_view_load_error_page          (EphyWebView               *view,
+                                                                   const char                *uri,
+                                                                   EphyWebViewErrorPage       page,
+                                                                   GError                    *error);
 /* These should be private */
 void                       ephy_web_view_set_address              (EphyWebView               *view,
                                                                    const char                *address);
@@ -191,11 +185,15 @@ void                       ephy_web_view_save                     (EphyWebView
                                                                    const char                *uri);
 void                       ephy_web_view_load_homepage            (EphyWebView               *view);
 
-char *
-ephy_web_view_create_web_application (EphyWebView *view, const char *title, GdkPixbuf *icon);
-
-GdkPixbuf *
-ephy_web_view_get_snapshot (EphyWebView *view, int x, int y, int width, int height);
+char *                     ephy_web_view_create_web_application   (EphyWebView               *view,
+                                                                   const char                *title,
+                                                                   GdkPixbuf                 *icon);
+GdkPixbuf *                ephy_web_view_get_snapshot             (EphyWebView               *view,
+                                                                   int                        x,
+                                                                   int                        y,
+                                                                   int                        width,
+                                                                   int                        height);
+gboolean                   ephy_web_view_is_loading_homepage      (EphyWebView               *view);
 
 G_END_DECLS
 
