@@ -31,7 +31,11 @@
 #include "totem-glow-button.h"
 
 #include <glib/gi18n.h>
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
+#endif
 
 G_DEFINE_TYPE (EphyDownloadWidget, ephy_download_widget, GTK_TYPE_BOX)
 
@@ -46,6 +50,8 @@ struct _EphyDownloadWidgetPrivate
   GtkWidget *button;
   GtkWidget *menu;
   GtkWidget *icon;
+
+  gboolean finished;
 };
 
 enum
@@ -72,6 +78,30 @@ get_gicon_from_download (EphyDownload *ephy_download)
   return gicon;
 }
 
+static char *
+get_destination_basename_from_download (EphyDownload *ephy_download)
+{
+  WebKitDownload *download;
+  const char *dest;
+  char *basename;
+  char *unescaped;
+
+  download = ephy_download_get_webkit_download (ephy_download);
+#ifdef HAVE_WEBKIT2
+  dest = NULL;
+#else
+  dest = webkit_download_get_destination_uri (download);
+#endif
+  if (!dest)
+    return NULL;
+
+  basename = g_filename_display_basename (dest);
+  unescaped = g_uri_unescape_string (basename, NULL);
+  g_free (basename);
+
+  return unescaped;
+}
+
 static char *
 format_interval (gdouble interval)
 {
@@ -99,6 +129,10 @@ format_interval (gdouble interval)
 static gdouble
 get_remaining_time (WebKitDownload *download)
 {
+#ifdef HAVE_WEBKIT2
+  /* TODO: Downloads */
+  return -1.0;
+#else
   gint64 total, cur;
   gdouble elapsed_time;
   gdouble remaining_time;
@@ -115,64 +149,109 @@ get_remaining_time (WebKitDownload *download)
   remaining_time = per_byte_time * (total - cur);
 
   return remaining_time;
+#endif
 }
 
 static void
 download_clicked_cb (GtkButton *button,
                      EphyDownloadWidget *widget)
 {
-  WebKitDownloadStatus status;
   EphyDownload *download;
 
-  download = widget->priv->download;
-  status = webkit_download_get_status (ephy_download_get_webkit_download (download));
-
-  if (status != WEBKIT_DOWNLOAD_STATUS_FINISHED)
+  if (!widget->priv->finished)
     return;
 
+  download = widget->priv->download;
   if (ephy_download_do_download_action (download, EPHY_DOWNLOAD_ACTION_AUTO))
     gtk_widget_destroy (GTK_WIDGET (widget));
 }
 
+#ifdef HAVE_WEBKIT2
+/* TODO: Downloads */
+#else
+static void
+update_download_icon (EphyDownloadWidget *widget)
+{
+  GIcon *new_icon, *old_icon;
+
+  new_icon = get_gicon_from_download (widget->priv->download);
+  gtk_image_get_gicon (GTK_IMAGE (widget->priv->icon), &old_icon, NULL);
+  if (!g_icon_equal (new_icon, old_icon)) {
+    gtk_image_set_from_gicon (GTK_IMAGE (widget->priv->icon), new_icon,
+                              GTK_ICON_SIZE_LARGE_TOOLBAR);
+  }
+  g_object_unref (new_icon);
+}
+
+static void
+update_download_label_and_tooltip (EphyDownloadWidget *widget,
+                                   const char *download_label)
+{
+  WebKitDownload *download;
+  char *remaining_tooltip;
+  char *destination;
+
+  download = ephy_download_get_webkit_download (widget->priv->download);
+  destination = g_filename_display_basename (webkit_download_get_destination_uri (download));
+
+  remaining_tooltip = g_markup_printf_escaped ("%s\n%s", destination, download_label);
+  g_free (destination);
+
+  gtk_label_set_text (GTK_LABEL (widget->priv->remaining), download_label);
+  gtk_widget_set_tooltip_text (GTK_WIDGET (widget), remaining_tooltip);
+  g_free (remaining_tooltip);
+}
+
+static gboolean
+download_content_length_is_known (WebKitDownload *download)
+{
+  WebKitNetworkResponse *response;
+  SoupMessage* message;
+
+  response = webkit_download_get_network_response (download);
+  message = webkit_network_response_get_message (response);
+  return soup_message_headers_get_content_length (message->response_headers) > 0;
+}
+
 static void
 widget_progress_cb (GObject *object,
                     GParamSpec *pspec,
                     EphyDownloadWidget *widget)
 {
   WebKitDownload *download;
-  char *destination;
-  gdouble time;
-  char *remaining;
   int progress;
-  char *remaining_label;
-  char *remaining_tooltip;
+  char *download_label = NULL;
 
   download = WEBKIT_DOWNLOAD (object);
-  destination = g_filename_display_basename (webkit_download_get_destination_uri (download));
   progress = webkit_download_get_progress (download) * 100;
 
   if (progress % 10 == 0)
-    gtk_image_set_from_gicon (GTK_IMAGE (widget->priv->icon),
-                              get_gicon_from_download (widget->priv->download),
-                              GTK_ICON_SIZE_LARGE_TOOLBAR);
+    update_download_icon (widget);
 
-  time = get_remaining_time (download);
+  if (download_content_length_is_known (download)) {
+    gdouble time;
 
-  if (time > 0)
-    remaining = format_interval (time);
-  else
-    remaining = g_strdup (_("Finished"));
+    time = get_remaining_time (download);
+    if (time > 0) {
+      char *remaining;
 
-  remaining_label = g_strdup_printf ("%d%% (%s)", progress, remaining);
-  remaining_tooltip = g_markup_printf_escaped ("%s\n%s", destination, remaining_label);
+      remaining = format_interval (time);
+      download_label = g_strdup_printf ("%d%% (%s)", progress, remaining);
+      g_free (remaining);
+    }
+  } else {
+    gint64 current_size;
 
-  gtk_label_set_text (GTK_LABEL (widget->priv->remaining), remaining_label);
-  gtk_widget_set_tooltip_text (GTK_WIDGET (widget), remaining_tooltip);
+    /* Unknown content length, show received bytes instead. */
+    current_size = webkit_download_get_current_size (download);
+    if (current_size > 0)
+      download_label = g_format_size (current_size);
+  }
 
-  g_free (destination);
-  g_free (remaining);
-  g_free (remaining_label);
-  g_free (remaining_tooltip);
+  if (download_label) {
+    update_download_label_and_tooltip (widget, download_label);
+    g_free (download_label);
+  }
 }
 
 static void
@@ -184,8 +263,12 @@ widget_status_cb (GObject *object,
 
   status = webkit_download_get_status (WEBKIT_DOWNLOAD (object));
 
-  if (status == WEBKIT_DOWNLOAD_STATUS_FINISHED)
-    totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (widget->priv->button), TRUE);
+  if (status != WEBKIT_DOWNLOAD_STATUS_FINISHED)
+    return;
+
+  widget->priv->finished = TRUE;
+  update_download_label_and_tooltip (widget, _("Finished"));
+  totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (widget->priv->button), TRUE);
 }
 
 static gboolean
@@ -211,6 +294,7 @@ widget_error_cb (WebKitDownload *download,
 
   return FALSE;
 }
+#endif
 
 static void
 open_activate_cb (GtkMenuItem *item, EphyDownloadWidget *widget)
@@ -238,8 +322,9 @@ download_menu_clicked_cb (GtkWidget *button,
                           GdkEventButton *event,
                           EphyDownloadWidget *widget)
 {
-  WebKitDownloadStatus status;
-  gboolean finished;
+#ifdef HAVE_WEBKIT2
+  /* TODO: Downloads */
+#else
   GtkWidget *item;
   GtkWidget *menu;
   GtkWidget *box;
@@ -250,8 +335,6 @@ download_menu_clicked_cb (GtkWidget *button,
 
   download = ephy_download_get_webkit_download (widget->priv->download);
 
-  status = webkit_download_get_status (download);
-  finished = (status == WEBKIT_DOWNLOAD_STATUS_FINISHED);
   basename = g_filename_display_basename (webkit_download_get_destination_uri (download));
   name = g_uri_unescape_string (basename, NULL);
 
@@ -270,7 +353,7 @@ download_menu_clicked_cb (GtkWidget *button,
 
   item = gtk_menu_item_new_with_label (_("Cancel"));
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-  gtk_widget_set_sensitive (item, !finished);
+  gtk_widget_set_sensitive (item, !widget->priv->finished);
   g_signal_connect (item, "activate",
                     G_CALLBACK (cancel_activate_cb), widget);
 
@@ -279,13 +362,13 @@ download_menu_clicked_cb (GtkWidget *button,
 
   item = gtk_menu_item_new_with_label (_("Open"));
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-  gtk_widget_set_sensitive (item, finished);
+  gtk_widget_set_sensitive (item, widget->priv->finished);
   g_signal_connect (item, "activate",
                     G_CALLBACK (open_activate_cb), widget);
 
   item = gtk_menu_item_new_with_label (_("Show in folder"));
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-  gtk_widget_set_sensitive (item, finished);
+  gtk_widget_set_sensitive (item, widget->priv->finished);
   g_signal_connect (item, "activate",
                     G_CALLBACK (folder_activate_cb), widget);
 
@@ -294,6 +377,7 @@ download_menu_clicked_cb (GtkWidget *button,
   gtk_menu_attach_to_widget (GTK_MENU (menu), button, NULL);
   gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
                   event->button, event->time);
+#endif
 }
 
 static void
@@ -348,9 +432,13 @@ ephy_download_widget_dispose (GObject *object)
   if (widget->priv->download != NULL) {
     download = ephy_download_get_webkit_download (widget->priv->download);
 
+#ifdef HAVE_WEBKIT2
+    /* TODO: Downloads */
+#else
     g_signal_handlers_disconnect_by_func (download, widget_progress_cb, widget);
     g_signal_handlers_disconnect_by_func (download, widget_status_cb, widget);
     g_signal_handlers_disconnect_by_func (download, widget_error_cb, widget);
+#endif
 
     g_object_unref (widget->priv->download);
     widget->priv->download = NULL;
@@ -415,6 +503,21 @@ ephy_download_widget_get_download (EphyDownloadWidget *widget)
   return widget->priv->download;
 }
 
+/**
+ * ephy_download_widget_download_is_finished:
+ * @widget: an #EphyDownloadWidget
+ *
+ * Whether the download finished
+ *
+ * Returns: %TRUE if download operation finished or %FALSE otherwise
+ **/
+gboolean
+ephy_download_widget_download_is_finished (EphyDownloadWidget *widget)
+{
+  g_return_val_if_fail (EPHY_IS_DOWNLOAD_WIDGET (widget), FALSE);
+  return widget->priv->finished;
+}
+
 /**
  * ephy_download_widget_new:
  * @ephy_download: the #EphyDownload that @widget is wrapping
@@ -436,8 +539,9 @@ ephy_download_widget_new (EphyDownload *ephy_download)
   GtkWidget *menu;
   GtkWidget *remain;
 
-  char *dest, *basename;
+  char *dest;
   WebKitDownload *download;
+  GIcon *gicon;
 
   g_return_val_if_fail (EPHY_IS_DOWNLOAD (ephy_download), NULL);
 
@@ -445,16 +549,16 @@ ephy_download_widget_new (EphyDownload *ephy_download)
                          "download", ephy_download, NULL);
   download = ephy_download_get_webkit_download (ephy_download);
 
-  basename = g_filename_display_basename (webkit_download_get_destination_uri (download));
-  dest = g_uri_unescape_string (basename, NULL);
-
   grid = gtk_grid_new ();
 
   button = totem_glow_button_new ();
   menu = gtk_button_new ();
 
-  icon = gtk_image_new_from_gicon (get_gicon_from_download (ephy_download),
-                                   GTK_ICON_SIZE_LARGE_TOOLBAR);
+  gicon = get_gicon_from_download (ephy_download);
+  icon = gtk_image_new_from_gicon (gicon, GTK_ICON_SIZE_LARGE_TOOLBAR);
+  g_object_unref (gicon);
+
+  dest = get_destination_basename_from_download (ephy_download);
   text = gtk_label_new (dest);
   gtk_misc_set_alignment (GTK_MISC (text), 0, 0.5);
   gtk_label_set_ellipsize (GTK_LABEL (text), PANGO_ELLIPSIZE_END);
@@ -468,8 +572,6 @@ ephy_download_widget_new (EphyDownload *ephy_download)
   gtk_grid_attach (GTK_GRID (grid), remain, 1, 1, 1, 1);
 
   gtk_widget_set_tooltip_text (GTK_WIDGET (widget), dest);
-
-  g_free (basename);
   g_free (dest);
 
   widget->priv->icon = icon;
@@ -477,12 +579,16 @@ ephy_download_widget_new (EphyDownload *ephy_download)
   widget->priv->remaining = remain;
   widget->priv->menu = menu;
 
+#ifdef HAVE_WEBKIT2
+  /* TODO: Downloads */
+#else
   g_signal_connect (download, "notify::progress",
                     G_CALLBACK (widget_progress_cb), widget);
   g_signal_connect (download, "notify::status",
                     G_CALLBACK (widget_status_cb), widget);
   g_signal_connect (download, "error",
                     G_CALLBACK (widget_error_cb), widget);
+#endif
 
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_HALF);
   gtk_button_set_relief (GTK_BUTTON (menu), GTK_RELIEF_NORMAL);
