@@ -38,7 +38,11 @@
 #include "ephy-window.h"
 
 #include <gtk/gtk.h>
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
+#endif
 
 #define EPHY_NAVIGATION_HISTORY_ACTION_GET_PRIVATE(object)		\
   (G_TYPE_INSTANCE_GET_PRIVATE ((object),				\
@@ -112,15 +116,25 @@ action_activate (GtkAction *action)
     if (ephy_gui_is_middle_click () ||
         ephy_link_action_get_button (EPHY_LINK_ACTION (history_action)) == 2) {
       const char *forward_uri;
+#ifdef HAVE_WEBKIT2
+      WebKitBackForwardList *history;
+      WebKitBackForwardListItem *forward_item;
+#else
       WebKitWebHistoryItem *forward_item;
       WebKitWebBackForwardList *history;
+#endif
 
       /* Forward history is not copied when opening
          a new tab, so get the forward URI manually
          and load it */
       history = webkit_web_view_get_back_forward_list (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
+#ifdef HAVE_WEBKIT2
+      forward_item = webkit_back_forward_list_get_forward_item (history);
+      forward_uri = webkit_back_forward_list_item_get_original_uri (forward_item);
+#else
       forward_item = webkit_web_back_forward_list_get_forward_item (history);
       forward_uri = webkit_web_history_item_get_original_uri (forward_item);
+#endif
 
       embed = ephy_shell_new_tab (ephy_shell_get_default (),
                                   EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
@@ -210,8 +224,7 @@ item_enter_notify_event_cb (GtkWidget *widget,
   char *text;
 
   text = g_object_get_data (G_OBJECT (widget), "link-message");
-  ephy_web_view_set_link_message (view, g_strdup (text));
-  g_object_set_data (G_OBJECT (widget), "link-message", text);
+  ephy_web_view_set_link_message (view, text);
 
   return FALSE;
 }
@@ -225,6 +238,9 @@ item_leave_notify_event_cb (GtkWidget *widget,
   return FALSE;
 }
 
+#ifdef HAVE_WEBKIT2
+/* TODO: favicons */
+#else
 static void
 icon_loaded_cb (GObject *source,
                 GAsyncResult *result,
@@ -246,6 +262,7 @@ icon_loaded_cb (GObject *source,
     g_object_unref (favicon);
   }
 }
+#endif
 
 static GtkWidget *
 new_history_menu_item (EphyWebView *view,
@@ -254,8 +271,12 @@ new_history_menu_item (EphyWebView *view,
 {
   GtkWidget *item;
   GtkLabel *label;
+#ifdef HAVE_WEBKIT2
+  /* TODO: favicons */
+#else
   WebKitFaviconDatabase* database;
   GdkPixbuf *favicon;
+#endif
 
   g_return_val_if_fail (address != NULL && origtext != NULL, NULL);
 
@@ -264,7 +285,9 @@ new_history_menu_item (EphyWebView *view,
   label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (item)));
   gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
   gtk_label_set_max_width_chars (label, MAX_LABEL_LENGTH);
-
+#ifdef HAVE_WEBKIT2
+  /* TODO: favicons */
+#else
   database = webkit_get_favicon_database ();
   favicon = webkit_favicon_database_try_get_favicon_pixbuf (database, address,
                                                             FAVICON_SIZE, FAVICON_SIZE);
@@ -283,8 +306,9 @@ new_history_menu_item (EphyWebView *view,
                                                 (GAsyncReadyCallback) icon_loaded_cb,
                                                 GTK_IMAGE_MENU_ITEM (item));
   }
+#endif
 
-  g_object_set_data (G_OBJECT (item), "link-message", g_strdup (address));
+  g_object_set_data_full (G_OBJECT (item), "link-message", g_strdup (address), (GDestroyNotify) g_free);
 
   g_signal_connect (item, "enter-notify-event",
                     G_CALLBACK (item_enter_notify_event_cb), view);
@@ -301,6 +325,9 @@ set_new_back_history (EphyEmbed *source,
                       EphyEmbed *dest,
                       gint offset)
 {
+#ifdef HAVE_WEBKIT2
+  /* TODO: WebKitBackForwardList: In WebKit2 WebKitBackForwardList can't be modified */
+#else
   WebKitWebView *source_view, *dest_view;
   WebKitWebBackForwardList* source_list, *dest_list;
   WebKitWebHistoryItem *item;
@@ -338,16 +365,23 @@ set_new_back_history (EphyEmbed *source,
     items = items->next;
   }
   g_list_free (items);
+#endif
 }
 
 static void
 middle_click_handle_on_history_menu_item (EphyNavigationHistoryAction *action,
                                           EphyEmbed *embed,
+#ifdef HAVE_WEBKIT2
+                                          WebKitBackForwardListItem *item)
+#else
                                           WebKitWebHistoryItem *item)
+#endif
 {
   EphyEmbed *new_embed = NULL;
   WebKitWebView *web_view;
+#ifndef HAVE_WEBKIT2
   WebKitWebBackForwardList *history;
+#endif
   GList *list;
   const gchar *url;
   guint current;
@@ -355,6 +389,10 @@ middle_click_handle_on_history_menu_item (EphyNavigationHistoryAction *action,
 
   web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
 
+#ifdef HAVE_WEBKIT2
+  /* TODO: WebKitBackForwardList is read-only in WebKit2 */
+  offset = 0;
+#else
   /* Save old history and item's offset from current */
   history = webkit_web_view_get_back_forward_list (web_view);
   if (action->priv->direction == EPHY_NAVIGATION_HISTORY_DIRECTION_BACK) {
@@ -367,6 +405,7 @@ middle_click_handle_on_history_menu_item (EphyNavigationHistoryAction *action,
     current = g_list_length (list);
   }
   offset = current - g_list_index (list, item);
+#endif
 
   new_embed = ephy_shell_new_tab (ephy_shell_get_default (),
                                   EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
@@ -382,7 +421,11 @@ middle_click_handle_on_history_menu_item (EphyNavigationHistoryAction *action,
   set_new_back_history (embed, new_embed, offset);
 
   /* Load the new URL */
+#ifdef HAVE_WEBKIT2
+  url = webkit_back_forward_list_item_get_original_uri (item);
+#else
   url = webkit_web_history_item_get_original_uri (item);
+#endif
   ephy_web_view_load_url (ephy_embed_get_web_view (new_embed), url);
 }
 
@@ -390,7 +433,11 @@ static void
 activate_menu_item_cb (GtkWidget *menuitem,
                        EphyNavigationHistoryAction *action)
 {
+#ifdef HAVE_WEBKIT2
+  WebKitBackForwardListItem *item;
+#else
   WebKitWebHistoryItem *item;
+#endif
   EphyWindow *window;
   EphyEmbed *embed;
 
@@ -398,7 +445,11 @@ activate_menu_item_cb (GtkWidget *menuitem,
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
   g_return_if_fail (embed != NULL);
 
+#ifdef HAVE_WEBKIT2
+  item = (WebKitBackForwardListItem *) g_object_get_data (G_OBJECT (menuitem), HISTORY_ITEM_DATA_KEY);
+#else
   item = (WebKitWebHistoryItem *) g_object_get_data (G_OBJECT (menuitem), HISTORY_ITEM_DATA_KEY);
+#endif
   g_return_if_fail (item != NULL);
 
   if (ephy_gui_is_middle_click ())
@@ -407,14 +458,27 @@ activate_menu_item_cb (GtkWidget *menuitem,
     WebKitWebView *web_view;
 
     web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+#ifdef HAVE_WEBKIT2
+    webkit_web_view_go_to_back_forward_list_item (web_view, item);
+#else
     webkit_web_view_go_to_back_forward_item (web_view, item);
+#endif
   }
 }
-static GList*
-webkit_construct_history_list (WebKitWebView *web_view,
+
+static GList *
+construct_webkit_history_list (WebKitWebView *web_view,
                                WebKitHistoryType hist_type,
                                int limit)
 {
+#ifdef HAVE_WEBKIT2
+  WebKitBackForwardList *back_forward_list;
+
+  back_forward_list = webkit_web_view_get_back_forward_list (web_view);
+  return hist_type == WEBKIT_HISTORY_FORWARD ?
+    g_list_reverse (webkit_back_forward_list_get_forward_list_with_limit (back_forward_list, limit)) :
+    webkit_back_forward_list_get_back_list_with_limit (back_forward_list, limit);
+#else
   WebKitWebBackForwardList *web_back_forward_list;
   GList *webkit_items;
 
@@ -428,6 +492,7 @@ webkit_construct_history_list (WebKitWebView *web_view,
                                                                           limit);
 
   return webkit_items;
+#endif
 }
 
 static GtkWidget *
@@ -448,22 +513,33 @@ build_dropdown_menu (EphyNavigationHistoryAction *action)
   web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
 
   if (action->priv->direction == EPHY_NAVIGATION_HISTORY_DIRECTION_BACK)
-    list = webkit_construct_history_list (web_view,
+    list = construct_webkit_history_list (web_view,
                                           WEBKIT_HISTORY_BACKWARD, 10);
   else
-    list = webkit_construct_history_list (web_view,
+    list = construct_webkit_history_list (web_view,
                                           WEBKIT_HISTORY_FORWARD, 10);
 
   for (l = list; l != NULL; l = l->next) {
     GtkWidget *item;
+#ifdef HAVE_WEBKIT2
+    WebKitBackForwardListItem *hitem;
+#else
     WebKitWebHistoryItem *hitem;
+#endif
     const char *uri;
     char *title;
 
+#ifdef HAVE_WEBKIT2
+    hitem = (WebKitBackForwardListItem *) l->data;
+    uri =  webkit_back_forward_list_item_get_uri (hitem);
+
+    title = g_strdup (webkit_back_forward_list_item_get_title (hitem));
+#else
     hitem = (WebKitWebHistoryItem *) l->data;
     uri = webkit_web_history_item_get_uri (hitem);
 
     title = g_strdup (webkit_web_history_item_get_title (hitem));
+#endif
 
     if (title == NULL || g_strstrip (title)[0] == '\0')
       item = new_history_menu_item (EPHY_WEB_VIEW (web_view), uri, uri);
