@@ -22,7 +22,6 @@
 #include "config.h"
 #include "ephy-session.h"
 
-#include "eggdesktopfile.h"
 #include "ephy-bookmarks-editor.h"
 #include "ephy-debug.h"
 #include "ephy-embed-container.h"
@@ -32,9 +31,8 @@
 #include "ephy-file-helpers.h"
 #include "ephy-gui.h"
 #include "ephy-history-window.h"
-#include "ephy-notebook.h"
 #include "ephy-prefs.h"
-#include "ephy-request-about.h"
+#include "ephy-about-handler.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-window.h"
@@ -130,6 +128,16 @@ session_delete (EphySession *session,
 	g_object_unref (file);
 }
 
+#ifdef HAVE_WEBKIT2
+static void
+load_changed_cb (WebKitWebView *view,
+		 WebKitLoadEvent load_event,
+		 EphySession *session)
+{
+	if (!ephy_web_view_load_failed (EPHY_WEB_VIEW (view)))
+		ephy_session_save (session, SESSION_STATE);
+}
+#else
 static void
 load_status_notify_cb (EphyWebView *view,
 		       GParamSpec *pspec,
@@ -144,6 +152,7 @@ load_status_notify_cb (EphyWebView *view,
 	    status == WEBKIT_LOAD_FINISHED)
 		ephy_session_save (session, SESSION_STATE);
 }
+#endif
 
 static void
 notebook_page_added_cb (GtkWidget *notebook,
@@ -151,8 +160,13 @@ notebook_page_added_cb (GtkWidget *notebook,
 			guint position,
 			EphySession *session)
 {
+#ifdef HAVE_WEBKIT2
+	g_signal_connect (ephy_embed_get_web_view (embed), "load-changed",
+			  G_CALLBACK (load_changed_cb), session);
+#else
 	g_signal_connect (ephy_embed_get_web_view (embed), "notify::load-status",
 			  G_CALLBACK (load_status_notify_cb), session);
+#endif
 }
 
 static void
@@ -163,9 +177,15 @@ notebook_page_removed_cb (GtkWidget *notebook,
 {
 	ephy_session_save (session, SESSION_STATE);
 
+#ifdef HAVE_WEBKIT2
+	g_signal_handlers_disconnect_by_func
+		(ephy_embed_get_web_view (embed), G_CALLBACK (load_changed_cb),
+		 session);
+#else
 	g_signal_handlers_disconnect_by_func
 		(ephy_embed_get_web_view (embed), G_CALLBACK (load_status_notify_cb),
 		 session);
+#endif
 }
 
 static void
@@ -327,7 +347,11 @@ session_command_open_uris (EphySession *session,
 	{
 		const char *url = uris[i];
 		EphyNewTabFlags page_flags;
+#ifdef HAVE_WEBKIT2
+		WebKitURIRequest *request = NULL;
+#else
 		WebKitNetworkRequest *request = NULL;
+#endif
 
 		if (url[0] == '\0')
 		{
@@ -336,7 +360,11 @@ session_command_open_uris (EphySession *session,
 		else
 		{
 			page_flags = EPHY_NEW_TAB_OPEN_PAGE;
+#ifdef HAVE_WEBKIT2
+			request = webkit_uri_request_new (url);
+#else
 			request = webkit_network_request_new (url);
+#endif
 		}
 
 		/* For the first URI, if we have a valid recovery
@@ -652,6 +680,7 @@ write_tab (xmlTextWriterPtr writer,
 	   EphyEmbed *embed)
 {
 	const char *address, *title;
+	char *new_address = NULL;
 	int ret;
 
 	ret = xmlTextWriterStartElement (writer, (xmlChar *) "embed");
@@ -662,10 +691,11 @@ write_tab (xmlTextWriterPtr writer,
 	 * loading. */
 	if (g_str_has_prefix (address, EPHY_ABOUT_SCHEME))
 	{
-		address = g_strconcat ("about", address + EPHY_ABOUT_SCHEME_LEN, NULL);
+		new_address = g_strconcat ("about", address + EPHY_ABOUT_SCHEME_LEN, NULL);
 	}
 	ret = xmlTextWriterWriteAttribute (writer, (xmlChar *) "url",
-					   (const xmlChar *) address);
+					   (const xmlChar *) (new_address ? new_address : address));
+	g_free (new_address);
 	if (ret < 0) return ret;
 
 	title = ephy_web_view_get_title (ephy_embed_get_web_view (embed));
@@ -917,10 +947,12 @@ confirm_before_recover (EphyWindow* window, char* url, char* title)
 static void 
 parse_embed (xmlNodePtr child,
 	     EphyWindow *window,
-	     gboolean is_first_window,
 	     EphySession *session)
 {
 	EphySessionPrivate *priv = session->priv;
+	gboolean is_first_window;
+
+	is_first_window = window == EPHY_WINDOW (priv->resume_window);
 
 	while (child != NULL)
 	{
@@ -932,7 +964,8 @@ parse_embed (xmlNodePtr child,
 
 			g_return_if_fail (window != NULL);
 
-			/* Check if that tab wasn't fully loaded yet when the session crashed */
+			/* Check if that tab wasn't fully loaded yet
+			 * when the session crashed. */
 			attr = xmlGetProp (child, (const xmlChar *) "loading");
 			was_loading = attr != NULL &&
 				      xmlStrEqual (attr, (const xmlChar *) "true");
@@ -942,26 +975,23 @@ parse_embed (xmlNodePtr child,
 			if (url == NULL) 
 				continue;
 
-			/* in the case that crash happens before we receive the URL from the server,
-			   this will open an about:blank tab. See http://bugzilla.gnome.org/show_bug.cgi?id=591294
-
-			   Otherwise, if the web was fully loaded, it is reloaded again.
-			
-			*/
+			/* In the case that crash happens before we receive the URL from the server, this will
+			   open an about:blank tab. See http://bugzilla.gnome.org/show_bug.cgi?id=591294
+			   Otherwise, if the web was fully loaded, it is reloaded again. */
 			if (!was_loading ||
 			    strcmp ((const char *) url, "about:blank") == 0)
 			{
 				recover_url = (char *) url;
 				
-				/* Reuse the window holding the recovery infobar instead of creating a new one */
-				if (is_first_window == TRUE && priv->resume_window != NULL)
+				/* Reuse the window holding the recovery infobar instead of creating a new one. */
+				if (is_first_window == TRUE && window != NULL)
 				{
 					EphyWebView *web_view;
 					EphyEmbed *embed;
 
-					embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (priv->resume_window));
+					embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 					web_view = ephy_embed_get_web_view (embed);
-					ephy_web_view_load_url(web_view, recover_url);
+					ephy_web_view_load_url (web_view, recover_url);
 
 					is_first_window = FALSE;
 				}
@@ -976,10 +1006,9 @@ parse_embed (xmlNodePtr child,
 			else if (was_loading && url != NULL &&
 				 strcmp ((const char *) url, "about:blank") != 0)
 			{
-				/* shows a message to the user that warns that this page was
+				/* Shows a message to the user that warns that this page was
 				   loading during crash and make Epiphany crash again,
-				   in this case we know the URL */
-
+				   in this case we know the URL. */
 				xmlChar* title = xmlGetProp (child, (const xmlChar *) "title");
 			
 				confirm_before_recover (window, (char*) url, (char*) title);
@@ -1053,37 +1082,41 @@ restore_geometry (GtkWindow *window,
 }
 
 /**
- * ephy_session_load:
- * @session: a #EphySession
- * @filename: the path of the source file
- * @user_time: a user_time, or 0
- *
- * Load a session from disk, restoring the windows and their state
- *
- * Return value: TRUE if at least a window has been opened
+ * ephy_session_load_from_string:
+ * @session: an #EphySession
+ * @session_data: a string with the session data to load
+ * @length: the length of @session_data, or -1 to assume %NULL terminated
+ * @user_time: a user time, or 0
+ * 
+ * Loads the session described in @session_data, restoring windows and
+ * their state.
+ * 
+ * Returns: TRUE if at least a window has been opened
  **/
 gboolean
-ephy_session_load (EphySession *session,
-		   const char *filename,
-		   guint32 user_time)
+ephy_session_load_from_string (EphySession *session,
+			       const char *session_data,
+			       gssize length,
+			       guint32 user_time)
 {
-	EphySessionPrivate *priv = session->priv;
+	EphySessionPrivate *priv;
 	xmlDocPtr doc;
 	xmlNodePtr child;
 	EphyWindow *window;
 	GtkWidget *widget = NULL;
-	GFile *save_to_file;
-	char *save_to_path;
 	gboolean first_window_created = FALSE;
+	
+	g_return_val_if_fail (EPHY_IS_SESSION (session), FALSE);
+	g_return_val_if_fail (session_data, FALSE);
 
-	LOG ("ephy_sesion_load %s", filename);
+	priv = session->priv;
 
-	save_to_file = get_session_file (filename);
-	save_to_path = g_file_get_path (save_to_file);
-	g_object_unref (save_to_file);
+	/* If length is -1 assume the data is a NULL-terminated, UTF-8
+	 * encoded string. */
+	if (length == -1)
+		length = g_utf8_strlen (session_data, -1);
 
-	doc = xmlParseFile (save_to_path);
-	g_free (save_to_path);
+	doc = xmlParseMemory (session_data, (int)length);
 
 	if (doc == NULL)
 	{
@@ -1120,8 +1153,7 @@ ephy_session_load (EphySession *session,
 			ephy_gui_window_update_user_time (widget, user_time);
 
 			/* Now add the tabs */
-			parse_embed (child->children, window,
-				     window == EPHY_WINDOW (priv->resume_window), session);
+			parse_embed (child->children, window, session);
 
 			/* Set focus to something sane */
 			tmp = xmlGetProp (child, (xmlChar *) "active-tab");
@@ -1140,9 +1172,12 @@ ephy_session_load (EphySession *session,
 				}
 			}
 
-			active_child = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-			gtk_widget_grab_focus (GTK_WIDGET (active_child));
-			gtk_widget_show (widget);
+			if (ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_TEST)
+			{
+				active_child = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+				gtk_widget_grab_focus (GTK_WIDGET (active_child));
+				gtk_widget_show (widget);
+			}
 		}
 		else if (xmlStrEqual (child->name, (const xmlChar *) "toolwindow"))
 		{
@@ -1191,6 +1226,54 @@ ephy_session_load (EphySession *session,
 	return (priv->windows != NULL || priv->tool_windows != NULL);
 }
 
+/**
+ * ephy_session_load:
+ * @session: a #EphySession
+ * @filename: the path of the source file
+ * @user_time: a user_time, or 0
+ *
+ * Load a session from disk, restoring the windows and their state
+ *
+ * Return value: TRUE if at least a window has been opened
+ **/
+gboolean
+ephy_session_load (EphySession *session,
+		   const char *filename,
+		   guint32 user_time)
+{
+	GFile *save_to_file;
+	char *save_to_path;
+	gboolean ret_value;
+	char *contents;
+	gsize length;
+	GError *error = NULL;
+
+	g_return_val_if_fail (EPHY_IS_SESSION (session), FALSE);
+	g_return_val_if_fail (filename, FALSE);
+
+	LOG ("ephy_sesion_load %s", filename);
+
+	save_to_file = get_session_file (filename);
+	save_to_path = g_file_get_path (save_to_file);
+	g_object_unref (save_to_file);
+
+	if (!g_file_get_contents (save_to_path, &contents, &length, &error))
+	{
+		LOG ("Could not load session, error reading session file: %s", error->message);
+		g_error_free (error);
+		g_free (save_to_path);
+
+		return FALSE;
+	}
+
+	ret_value = ephy_session_load_from_string (session, contents, length, user_time);
+
+	g_free (contents);
+	g_free (save_to_path);
+
+	return ret_value;
+}
+
 /**
  * ephy_session_get_windows:
  * @session: the #EphySession
