@@ -37,7 +37,6 @@
 #include "ephy-lockdown.h"
 #include "ephy-prefs.h"
 #include "ephy-private.h"
-#include "ephy-profile-utils.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-type-builtins.h"
@@ -264,27 +263,8 @@ ephy_shell_startup (GApplication* application)
   G_APPLICATION_CLASS (ephy_shell_parent_class)->startup (application);
 
   /* We're not remoting; start our services */
-  /* Migrate profile if we are not running a private instance */
-  /* TODO: we want to migrate each WebApp profile too */
   mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (application));
 
-  if (mode == EPHY_EMBED_SHELL_MODE_BROWSER) {
-    if (ephy_profile_utils_get_migration_version () < EPHY_PROFILE_MIGRATION_VERSION) {
-      GError *error = NULL;
-      char *argv[1] = { "ephy-profile-migrator" };
-      char *envp[1] = { "EPHY_LOG_MODULES=ephy-profile" };
-        
-      g_spawn_sync (NULL, argv, envp, G_SPAWN_SEARCH_PATH,
-                    NULL, NULL, NULL, NULL,
-                    NULL, &error);
-
-      if (error) {
-        LOG ("Failed to run migrator: %s", error->message);
-        g_error_free (error);
-      }
-    }
-  }
-
   if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION) {
     GtkBuilder *builder;
 
@@ -522,9 +502,8 @@ ephy_shell_new_window_cb (EphyEmbedSingle *single,
   LOG ("ephy_shell_new_window_cb tab chrome %d", chromemask);
 
   if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
-                              EPHY_PREFS_LOCKDOWN_JAVASCRIPT_CHROME)) {
+                              EPHY_PREFS_LOCKDOWN_JAVASCRIPT_CHROME))
     chromemask = EPHY_WEB_VIEW_CHROME_ALL;
-  }
 
   if (parent_embed != NULL) {
     /* this will either be a EphyWindow, or the embed itself
@@ -534,10 +513,9 @@ ephy_shell_new_window_cb (EphyEmbedSingle *single,
     parent = gtk_widget_get_toplevel (GTK_WIDGET (parent_embed));
   }
 
-  /* what's a popup ? ATM, any window opened with menubar toggled on
-   * is *not* a popup
+  /* Any window opened with a toolbar is *not* a popup.
    */
-  is_popup = (chromemask & EPHY_WEB_VIEW_CHROME_MENUBAR) == 0;
+  is_popup = (chromemask & EPHY_WEB_VIEW_CHROME_TOOLBAR) == 0;
 
   return ephy_shell_new_tab_full
     (shell,
@@ -691,7 +669,11 @@ EphyEmbed *
 ephy_shell_new_tab_full (EphyShell *shell,
                          EphyWindow *parent_window,
                          EphyEmbed *previous_embed,
+#ifdef HAVE_WEBKIT2
+                         WebKitURIRequest *request,
+#else
                          WebKitNetworkRequest *request,
+#endif
                          EphyNewTabFlags flags,
                          EphyWebViewChrome chrome,
                          gboolean is_popup,
@@ -699,40 +681,37 @@ ephy_shell_new_tab_full (EphyShell *shell,
 {
   EphyWindow *window;
   EphyEmbed *embed = NULL;
+  gboolean fullscreen_lockdown = FALSE;
   gboolean in_new_window = TRUE;
   gboolean open_page = FALSE;
-  gboolean jump_to;
+  gboolean jump_to = FALSE;
   gboolean active_is_blank = FALSE;
   gboolean copy_history = TRUE;
-  GtkWidget *nb;
-  int position = -1;
   gboolean is_empty = FALSE;
+  int position = -1;
 
   if (flags & EPHY_NEW_TAB_OPEN_PAGE) open_page = TRUE;
   if (flags & EPHY_NEW_TAB_IN_NEW_WINDOW) in_new_window = TRUE;
   if (flags & EPHY_NEW_TAB_IN_EXISTING_WINDOW) in_new_window = FALSE;
   if (flags & EPHY_NEW_TAB_DONT_COPY_HISTORY) copy_history = FALSE;
+  if (flags & EPHY_NEW_TAB_JUMP) jump_to = TRUE;
 
-  in_new_window = in_new_window &&
-    !g_settings_get_boolean
-    (EPHY_SETTINGS_LOCKDOWN,
-     EPHY_PREFS_LOCKDOWN_FULLSCREEN);
+  fullscreen_lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+                                                EPHY_PREFS_LOCKDOWN_FULLSCREEN);
+  in_new_window = in_new_window && !fullscreen_lockdown;
   g_return_val_if_fail (open_page == (gboolean)(request != NULL), NULL);
 
-  jump_to = (flags & EPHY_NEW_TAB_JUMP) != 0;
-
   LOG ("Opening new tab parent-window %p parent-embed %p in-new-window:%s jump-to:%s",
        parent_window, previous_embed, in_new_window ? "t" : "f", jump_to ? "t" : "f");
 
-  if (!in_new_window && parent_window != NULL) {
+  if (!in_new_window && parent_window != NULL)
     window = parent_window;
-  } else {
+  else
     window = ephy_window_new_with_chrome (chrome, is_popup);
-  }
 
   if (flags & EPHY_NEW_TAB_APPEND_AFTER) {
     if (previous_embed) {
-      nb = ephy_window_get_notebook (window);
+      GtkWidget *nb = ephy_window_get_notebook (window);
       /* FIXME this assumes the tab is the  direct notebook child */
       position = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
                                         GTK_WIDGET (previous_embed)) + 1;
@@ -741,7 +720,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
   }
 
   if (flags & EPHY_NEW_TAB_FROM_EXTERNAL) {
-    /* If the active embed is blank, us e that to open the url and jump to it */
+    /* If the active embed is blank, use that to open the url and jump to it */
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
     if (embed != NULL) {
       if (ephy_web_view_get_is_blank (ephy_embed_get_web_view (embed)) &&
@@ -766,7 +745,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
 
   ephy_gui_window_update_user_time (GTK_WIDGET (window), user_time);
 
-  if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0) {
+  if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0 &&
+      ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_TEST) {
     gtk_widget_show (GTK_WIDGET (window));
   }
 
@@ -785,7 +765,11 @@ ephy_shell_new_tab_full (EphyShell *shell,
     ephy_web_view_load_request (ephy_embed_get_web_view (embed),
                                 request);
 
+#ifdef HAVE_WEBKIT2
+    is_empty = ephy_embed_utils_url_is_empty (webkit_uri_request_get_uri (request));
+#else
     is_empty = ephy_embed_utils_url_is_empty (webkit_network_request_get_uri (request));
+#endif
   }
 
   /* Make sure the initial focus is somewhere sensible and not, for
@@ -797,7 +781,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
     if (is_empty) {
       /* empty page, focus location entry */
       ephy_window_activate_location (window);
-    } else if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0 && embed != NULL) {
+    } else if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0 && embed != NULL &&
+               ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_TEST) {
       /* non-empty page, focus the page. but make sure the widget is realised first! */
       gtk_widget_realize (GTK_WIDGET (embed));
       gtk_widget_grab_focus (GTK_WIDGET (embed));
@@ -827,7 +812,11 @@ ephy_shell_new_tab (EphyShell *shell,
                     EphyNewTabFlags flags)
 {
   EphyEmbed *embed;
+#ifdef HAVE_WEBKIT2
+  WebKitURIRequest *request = url ? webkit_uri_request_new (url) : NULL;
+#else
   WebKitNetworkRequest *request = url ? webkit_network_request_new (url) : NULL;
+#endif
 
   embed = ephy_shell_new_tab_full (shell, parent_window,
                                    previous_embed, request, flags,
