@@ -34,6 +34,7 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
+#include <libsoup/soup.h>
 #include <string.h>
 
 #define HTML_STRING "testing-ephy-web-view"
@@ -62,6 +63,28 @@ server_callback (SoupServer *server,
   soup_message_body_complete (msg->response_body);
 }
 
+#ifdef HAVE_WEBKIT2
+static void
+load_changed_cb (WebKitWebView *view, WebKitLoadEvent load_event, GMainLoop *loop)
+{
+  char *expected_url;
+  const char *loaded_url;
+
+  if (load_event != WEBKIT_LOAD_FINISHED)
+    return;
+
+  expected_url = g_object_get_data (G_OBJECT (view), "test.expected_url");
+  g_assert (expected_url != NULL);
+
+  loaded_url = webkit_web_view_get_uri (view);
+  g_assert_cmpstr (loaded_url, ==, expected_url);
+
+  g_signal_handlers_disconnect_by_func (view, load_changed_cb, loop);
+
+  g_free (expected_url);
+  g_main_loop_quit (loop);
+}
+#else
 static void
 notify_load_status_cb (WebKitWebView *view, GParamSpec *spec, GMainLoop *loop)
 {
@@ -85,6 +108,7 @@ notify_load_status_cb (WebKitWebView *view, GParamSpec *spec, GMainLoop *loop)
   g_free (expected_url);
   g_main_loop_quit (loop);
 }
+#endif
 
 typedef struct {
   const char *url;
@@ -156,8 +180,13 @@ test_ephy_web_view_load_url ()
 
     g_test_message ("[%s] \t-> %s", test.url, test.expected_url);
 
+#ifdef HAVE_WEBKIT2
+    g_signal_connect (view, "load-changed",
+                      G_CALLBACK (load_changed_cb), loop);
+#else
     g_signal_connect (view, "notify::load-status",
                       G_CALLBACK (notify_load_status_cb), loop);
+#endif
 
     g_main_loop_run (loop);
     g_main_loop_unref (loop);
@@ -246,7 +275,9 @@ main (int argc, char *argv[])
   ephy_debug_init ();
   ephy_embed_prefs_init ();
 
-  if (!ephy_file_helpers_init (NULL, TRUE, FALSE, NULL)) {
+  if (!ephy_file_helpers_init (NULL,
+                               EPHY_FILE_HELPERS_PRIVATE_PROFILE | EPHY_FILE_HELPERS_ENSURE_EXISTS,
+                               NULL)) {
     g_debug ("Something wrong happened with ephy_file_helpers_init()");
     return -1;
   }
